﻿//////////////////////////////////////////////
// SsaoEffect.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Camera ;
	class Program ;
	class Texture ;
}

/// Includes ---------------------------------

// nkAstraeus
#include "../../../Dll/DllDefines.h"

#include "../Effect.h"

#include "SsaoEffectInfo.h"

/// Class ------------------------------------

namespace nkAstraeus
{
	class DLL_ASTRAEUS_EXPORT SsaoEffect final : public Effect
	{
		private :

			// Attributes
			// Program
			nkGraphics::Program* _program ;
			SsaoEffectInfo _programInfo ;

			// Info
			SsaoEffectInfo _info ;

		private :

			// Functions
			// Program
			bool _prepareProgram () ;

			// Shader
			bool _prepareShader () ;

		public :

			// Constructor, destructor
			SsaoEffect () ;
			~SsaoEffect () ;

			// Getters
			virtual EFFECT_TYPE getType () const override ;
			nkGraphics::Texture* getPositionBuffer () const ;
			nkGraphics::Texture* getNormalBuffer () const ;
			nkGraphics::Texture* getDepthBuffer () const ;
			bool getLinearDepth () const ;
			nkGraphics::Camera* getCamera () const ;
			float getSamplingRadius () const ;
			float getMaxOcclusionDistance () const ;
			float getStrength () const ;
			float getDepthBias () const ;
			unsigned int getKernelSampleCount () const ;
			bool getOcclusionDrivesAlpha () const ;

			// Setters
			void setEffectFromGBufferInfo (nkGraphics::Texture* positionBuffer, nkGraphics::Texture* normalBuffer = nullptr) ;
			void setEffectFromDepthInfo (nkGraphics::Texture* depthBuffer, bool linearDepth = false) ;
			void setCamera (nkGraphics::Camera* value) ;
			void setSamplingRadius (float value) ;
			void setMaxOcclusionDistance (float value) ;
			void setStrength (float value) ;
			void setDepthBias (float value) ;
			void setKernelSampleCount (unsigned int value) ;
			void setOcclusionDrivesAlpha (bool value) ;

			// Load
			virtual bool load () override ;
			virtual void unload () override ;
	} ;
}